<?php
/**
 * @file
 * Contains \Drupal\masquerade\Access\SwitchAccessCheck.
 */

namespace Drupal\masquerade\Access;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\user\Entity\Role;

/**
 * Checks access for any masquerade permissions.
 */
class SwitchAccessCheck implements AccessInterface {

  /**
   * Check to see if user has any permissions to masquerade.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   Run access checks for this account.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function access(AccountInterface $account) {
    // Uid 1 may masquerade as anyone.
    if ($account->id() == 1) {
      return AccessResult::allowed()->cachePerUser();
    }
    // The current user must be allowed to masquerade.
    if ($account->hasPermission('masquerade as any user')) {
      return AccessResult::allowed()->cachePerUser();
    }
    // Permissions may be granted on a per-role basis.
    $account_roles = $account->getRoles();
    foreach ($account_roles as $role_id) {
      $role = Role::load($role_id);
      if (preg_match("/masquerade as \d+/", $role->id())) {
        return AccessResult::allowed()->cachePerUser();
      }
    }
    return AccessResult::neutral()->cachePerUser();
  }

}
